/*
 * Copyright (c) 2008-2016 Haulmont.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package com.haulmont.cuba.client;

import com.haulmont.cuba.core.global.AppBeans;
import com.haulmont.cuba.security.app.UserSessionService;
import com.haulmont.cuba.security.global.ClientBasedSession;
import com.haulmont.cuba.security.global.UserSession;

import java.io.Serializable;
import java.util.Locale;
import java.util.TimeZone;

/**
 * Client-side extension of {@link UserSession}.
 *
 * <p>Sends updates of the user session properties to the middleware if authenticated.</p>
 */
public class ClientUserSession extends UserSession implements ClientBasedSession {

    private static final long serialVersionUID = -5358664165808633540L;

    /**
     * Indicates whether the user has been authenticated. It is false for anonymous session.
     */
    protected volatile boolean authenticated = false;

    public ClientUserSession(UserSession src) {
        super(src);
    }

    @Override
    public void setAttribute(String name, Serializable value) {
        super.setAttribute(name, value);

        if (authenticated) {
            UserSessionService uss = AppBeans.get(UserSessionService.NAME);
            uss.setSessionAttribute(id, name, value);
        }
    }

    @Override
    public void removeAttribute(String name) {
        super.removeAttribute(name);

        if (authenticated) {
            UserSessionService uss = AppBeans.get(UserSessionService.NAME);
            uss.removeSessionAttribute(id, name);
        }
    }

    @Override
    public void setLocale(Locale locale) {
        super.setLocale(locale);

        if (authenticated) {
            UserSessionService uss = AppBeans.get(UserSessionService.NAME);
            uss.setSessionLocale(id, locale);
        }
    }

    @Override
    public void setTimeZone(TimeZone timeZone) {
        super.setTimeZone(timeZone);

        if (authenticated) {
            UserSessionService uss = AppBeans.get(UserSessionService.NAME);
            uss.setSessionTimeZone(id, timeZone);
        }
    }

    @Override
    public void setAddress(String address) {
        super.setAddress(address);

        if (authenticated) {
            UserSessionService uss = AppBeans.get(UserSessionService.NAME);
            uss.setSessionAddress(id, address);
        }
    }

    @Override
    public void setClientInfo(String clientInfo) {
        super.setClientInfo(clientInfo);

        if (authenticated) {
            UserSessionService uss = AppBeans.get(UserSessionService.NAME);
            uss.setSessionClientInfo(id, clientInfo);
        }
    }

    /**
     * @see #authenticated
     */
    public boolean isAuthenticated() {
        return authenticated;
    }

    /**
     * @see #authenticated
     */
    public void setAuthenticated(boolean authenticated) {
        this.authenticated = authenticated;
    }

    @Override
    public boolean hasRequestScopedInfo() {
        return !authenticated;
    }
}