/*
 * Copyright (c) 2008-2016 Haulmont.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package com.haulmont.chile.core.datatypes.impl;

import com.haulmont.chile.core.datatypes.Datatype;

import javax.annotation.Nonnull;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.ParseException;
import java.util.Locale;

public class StringDatatype implements Datatype<String> {

	public final static String NAME = "string";

    @Nonnull
    @Override
	public String format(Object value) {
		return value == null ? "" : (String) value;
	}

    @Nonnull
    @Override
    public String format(Object value, Locale locale) {
        return format(value);
    }

    @Override
	public Class getJavaClass() {
		return String.class;
	}

    @Override
	public String getName() {
		return NAME;
	}

    @Override
	public int getSqlType() {
		return Types.VARCHAR;
	}

    @Override
	public String parse(String value) {
		return value;
	}

    @Override
    public String parse(String value, Locale locale) throws ParseException {
        return value;
    }

    @Override
	public String read(ResultSet resultSet, int index) throws SQLException {
        return resultSet.getString(index);
	}

    @Override
	public void write(PreparedStatement statement, int index, Object value) throws SQLException {
		if (value == null) {
			statement.setString(index, null);
		} else {
			statement.setString(index, (String) value);
		}
	}

    @Override
    public String toString() {
        return NAME;
    }
}