/*
 * Copyright (c) 2008-2016 Haulmont.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package com.haulmont.cuba.core.sys.serialization;

import java.io.InputStream;
import java.io.OutputStream;

/**
 *
 *  Defines the contract for platform serialization implementations.
 */
public interface Serialization {
    /**
     * Serialize object {@code object} to stream {@code os}.
     * Stream isn't closed automatically after serialization.
     * User code should be close stream correctly
     *
     * @param object object
     * @param os output stream
     */
    void serialize(Object object, OutputStream os);

    /**
     * Deserialize object from stream
     */
    Object deserialize(InputStream is);

    /**
     * Serialize object to byte array
     */
    byte[] serialize(Object object);

    /**
     * Deserialize object from byte array
     */
    Object deserialize(byte[] bytes);
}
